@@ -13,17 +13,17 @@
 %            Read/Write Raw CCIR 601 4:1:1 or 4:2:2 Image Format              %
 %                                                                             %
 %                              Software Design                                %
-%                                John Cristy                                  %
+%                                   Cristy                                    %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -39,34 +39,35 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/cache.h"
-#include "magick/colorspace.h"
-#include "magick/constitute.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/geometry.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/resize.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/module.h"
-#include "magick/utility.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/colorspace.h"
+#include "MagickCore/constitute.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/geometry.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/resize.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/module.h"
+#include "MagickCore/utility.h"
 
 /*
   Forward declarations.
 */
 static MagickBooleanType
-  WriteYUVImage(const ImageInfo *,Image *);
+  WriteYUVImage(const ImageInfo *,Image *,ExceptionInfo *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -105,31 +106,27 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
   InterlaceType
     interlace;
 
-  long
+  MagickBooleanType
+    status;
+
+  ssize_t
+    x;
+
+  Quantum
+    *q;
+
+  unsigned char
+    *p;
+
+  ssize_t
+    count,
     horizontal_factor,
     vertical_factor,
     y;
 
-  MagickBooleanType
-    status;
-
-  register const PixelPacket
-    *chroma_pixels;
-
-  register long
-    x;
-
-  register PixelPacket
-    *q;
-
-  register long
-    i;
-
-  register unsigned char
-    *p;
-
-  ssize_t
-    count;
+  size_t
+    length,
+    quantum;
 
   unsigned char
     *scanline;
@@ -138,16 +135,19 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Allocate image structure.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
-  if (image_info->debug != MagickFalse)
+  assert(image_info->signature == MagickCoreSignature);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
-  assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  image=AcquireImage(image_info);
+  image=AcquireImage(image_info,exception);
   if ((image->columns == 0) || (image->rows == 0))
     ThrowReaderException(OptionError,"MustSpecifyImageSize");
-  image->depth=8;
+  status=SetImageExtent(image,image->columns,image->rows,exception);
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
+  quantum=(size_t) (image->depth <= 8 ? 1 : 2);
   interlace=image_info->interlace;
   horizontal_factor=2;
   vertical_factor=2;
@@ -160,12 +160,13 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         flags;
 
       flags=ParseGeometry(image_info->sampling_factor,&geometry_info);
-      horizontal_factor=(long) geometry_info.rho;
-      vertical_factor=(long) geometry_info.sigma;
-      if ((flags & SigmaValue) == 0)
-        vertical_factor=horizontal_factor;
-      if ((horizontal_factor != 1) && (horizontal_factor != 2) &&
-          (vertical_factor != 1) && (vertical_factor != 2))
+      if ((flags & RhoValue) != 0)
+        horizontal_factor=(ssize_t) geometry_info.rho;
+      vertical_factor=horizontal_factor;
+      if ((flags & SigmaValue) != 0)
+        vertical_factor=(ssize_t) geometry_info.sigma;
+      if (((horizontal_factor != 1) && (horizontal_factor != 2)) ||
+          ((vertical_factor != 1) && (vertical_factor != 2)))
         ThrowReaderException(CorruptImageError,"UnexpectedSamplingFactor");
     }
   if ((interlace == UndefinedInterlace) ||
@@ -186,99 +187,153 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
           image=DestroyImageList(image);
           return((Image *) NULL);
         }
-      for (i=0; i < image->offset; i++)
-        if (ReadBlobByte(image) == EOF)
-          {
-            ThrowFileException(exception,CorruptImageError,
-              "UnexpectedEndOfFile",image->filename);
-            break;
-          }
+      if (DiscardBlobBytes(image,(MagickSizeType) image->offset) == MagickFalse)
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
     }
   /*
     Allocate memory for a scanline.
   */
   if (interlace == NoInterlace)
-    scanline=(unsigned char *) AcquireQuantumMemory((size_t) 2UL*
-      image->columns+2UL,sizeof(*scanline));
+    scanline=(unsigned char *) AcquireQuantumMemory((size_t) (2UL*
+      image->columns+2UL),(size_t) quantum*sizeof(*scanline));
   else
-    scanline=(unsigned char *) AcquireQuantumMemory((size_t) image->columns,
-      sizeof(*scanline));
+    scanline=(unsigned char *) AcquireQuantumMemory(image->columns,
+      (size_t) quantum*sizeof(*scanline));
   if (scanline == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  status=MagickTrue;
   do
   {
-    chroma_image=CloneImage(image,image->columns/horizontal_factor,
-      image->rows/vertical_factor,MagickTrue,exception);
+    chroma_image=CloneImage(image,(size_t) ((ssize_t) image->columns+
+      (ssize_t) horizontal_factor-1)/(size_t) horizontal_factor,(size_t)
+      (image->rows+(size_t) vertical_factor-1)/
+      (size_t) vertical_factor,MagickTrue,exception);
     if (chroma_image == (Image *) NULL)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+      {
+        scanline=(unsigned char *) RelinquishMagickMemory(scanline); 
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+      }
     /*
       Convert raster image to pixel packets.
     */
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    if (SetImageExtent(image,0,0) == MagickFalse)
-      {
-        InheritException(exception,&image->exception);
-        return(DestroyImageList(image));
-      }
+    status=SetImageExtent(image,image->columns,image->rows,exception);
+    if (status == MagickFalse)
+      break;
     if (interlace == PartitionInterlace)
       {
         AppendImageFormat("Y",image->filename);
         status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
         if (status == MagickFalse)
           {
+            scanline=(unsigned char *) RelinquishMagickMemory(scanline); 
             image=DestroyImageList(image);
             return((Image *) NULL);
           }
       }
-    for (y=0; y < (long) image->rows; y++)
+    for (y=0; y < (ssize_t) image->rows; y++)
     {
-      register PixelPacket
+      Quantum
         *chroma_pixels;
 
       if (interlace == NoInterlace)
         {
           if ((y > 0) || (GetPreviousImageInList(image) == (Image *) NULL))
-            count=ReadBlob(image,(size_t) (2*image->columns),scanline);
+            {
+              length=2*quantum*image->columns;
+              count=ReadBlob(image,length,scanline);
+              if (count != (ssize_t) length)
+                {
+                  status=MagickFalse;
+                  ThrowFileException(exception,CorruptImageError,
+                    "UnexpectedEndOfFile",image->filename);
+                  break;
+                }
+            }
           p=scanline;
           q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-          if (q == (PixelPacket *) NULL)
+          if (q == (Quantum *) NULL)
             break;
           chroma_pixels=QueueAuthenticPixels(chroma_image,0,y,
             chroma_image->columns,1,exception);
-          if (chroma_pixels == (PixelPacket *) NULL)
+          if (chroma_pixels == (Quantum *) NULL)
             break;
-          for (x=0; x < (long) image->columns; x+=2)
+          for (x=0; x < (ssize_t) image->columns; x+=2)
           {
-            chroma_pixels->red=(Quantum) 0;
-            chroma_pixels->green=ScaleCharToQuantum(*p++);
-            q->red=ScaleCharToQuantum(*p++);
-            q->green=(Quantum) 0;
-            q->blue=(Quantum) 0;
-            q++;
-            q->green=0;
-            q->blue=0;
-            chroma_pixels->blue=ScaleCharToQuantum(*p++);
-            q->red=ScaleCharToQuantum(*p++);
-            chroma_pixels++;
-            q++;
+            SetPixelRed(chroma_image,0,chroma_pixels);
+            if (quantum == 1)
+              SetPixelGreen(chroma_image,ScaleCharToQuantum(*p++),
+                chroma_pixels);
+            else
+              {
+                SetPixelGreen(chroma_image,ScaleShortToQuantum(((*p) << 8) |
+                  *(p+1)),chroma_pixels);
+                p+=(ptrdiff_t) 2;
+              }
+            if (quantum == 1)
+              SetPixelRed(image,ScaleCharToQuantum(*p++),q);
+            else
+              {
+                SetPixelRed(image,ScaleShortToQuantum(((*p) << 8) | *(p+1)),q);
+                p+=(ptrdiff_t) 2;
+              }
+            SetPixelGreen(image,0,q);
+            SetPixelBlue(image,0,q);
+            q+=(ptrdiff_t) GetPixelChannels(image);
+            SetPixelGreen(image,0,q);
+            SetPixelBlue(image,0,q);
+            if (quantum == 1)
+              SetPixelBlue(chroma_image,ScaleCharToQuantum(*p++),chroma_pixels);
+            else
+              {
+                SetPixelBlue(chroma_image,ScaleShortToQuantum(((*p) << 8) |
+                  *(p+1)),chroma_pixels);
+                p+=(ptrdiff_t) 2;
+              }
+            if (quantum == 1)
+              SetPixelRed(image,ScaleCharToQuantum(*p++),q);
+            else
+              {
+                SetPixelRed(image,ScaleShortToQuantum(((*p) << 8) | *(p+1)),q);
+                p+=(ptrdiff_t) 2;
+              }
+            chroma_pixels+=(ptrdiff_t) GetPixelChannels(chroma_image);
+            q+=(ptrdiff_t) GetPixelChannels(image);
           }
         }
       else
         {
           if ((y > 0) || (GetPreviousImageInList(image) == (Image *) NULL))
-            count=ReadBlob(image,(size_t) image->columns,scanline);
+            {
+              length=quantum*image->columns;
+              count=ReadBlob(image,length,scanline);
+              if (count != (ssize_t) length)
+                {
+                  status=MagickFalse;
+                  ThrowFileException(exception,CorruptImageError,
+                    "UnexpectedEndOfFile",image->filename);
+                  break;
+                }
+            }
           p=scanline;
           q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-          if (q == (PixelPacket *) NULL)
+          if (q == (Quantum *) NULL)
             break;
-          for (x=0; x < (long) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
-            q->red=ScaleCharToQuantum(*p++);
-            q->green=0;
-            q->blue=0;
-            q++;
+            if (quantum == 1)
+              SetPixelRed(image,ScaleCharToQuantum(*p++),q);
+            else
+              {
+                SetPixelRed(image,ScaleShortToQuantum(((*p) << 8) | *(p+1)),q);
+                p+=(ptrdiff_t) 2;
+              }
+            SetPixelGreen(image,0,q);
+            SetPixelBlue(image,0,q);
+            q+=(ptrdiff_t) GetPixelChannels(image);
           }
         }
       if (SyncAuthenticPixels(image,exception) == MagickFalse)
@@ -288,63 +343,100 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
           break;
       if (image->previous == (Image *) NULL)
         {
-          status=SetImageProgress(image,LoadImageTag,y,image->rows);
+          status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+            image->rows);
           if (status == MagickFalse)
             break;
         }
     }
     if (interlace == PartitionInterlace)
       {
-        (void) CloseBlob(image);
+        if (CloseBlob(image) == MagickFalse)
+          break;
         AppendImageFormat("U",image->filename);
         status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
         if (status == MagickFalse)
           {
+            scanline=(unsigned char *) RelinquishMagickMemory(scanline); 
             image=DestroyImageList(image);
             return((Image *) NULL);
           }
       }
     if (interlace != NoInterlace)
       {
-        for (y=0; y < (long) chroma_image->rows; y++)
+        for (y=0; y < (ssize_t) chroma_image->rows; y++)
         {
-          count=ReadBlob(image,(size_t) chroma_image->columns,scanline);
+          length=quantum*chroma_image->columns;
+          count=ReadBlob(image,length,scanline);
+          if (count != (ssize_t) length)
+            {
+              status=MagickFalse;
+              ThrowFileException(exception,CorruptImageError,
+                "UnexpectedEndOfFile",image->filename);
+              break;
+            }
           p=scanline;
-          q=QueueAuthenticPixels(chroma_image,0,y,chroma_image->columns,1,exception);
-          if (q == (PixelPacket *) NULL)
+          q=QueueAuthenticPixels(chroma_image,0,y,chroma_image->columns,1,
+            exception);
+          if (q == (Quantum *) NULL)
             break;
-          for (x=0; x < (long) chroma_image->columns; x++)
+          for (x=0; x < (ssize_t) chroma_image->columns; x++)
           {
-            q->red=(Quantum) 0;
-            q->green=ScaleCharToQuantum(*p++);
-            q->blue=(Quantum) 0;
-            q++;
+            SetPixelRed(chroma_image,0,q);
+            if (quantum == 1)
+              SetPixelGreen(chroma_image,ScaleCharToQuantum(*p++),q);
+            else
+              {
+                SetPixelGreen(chroma_image,ScaleShortToQuantum(((*p) << 8) |
+                  *(p+1)),q);
+                p+=(ptrdiff_t) 2;
+              }
+            SetPixelBlue(chroma_image,0,q);
+            q+=(ptrdiff_t) GetPixelChannels(chroma_image);
           }
           if (SyncAuthenticPixels(chroma_image,exception) == MagickFalse)
             break;
         }
       if (interlace == PartitionInterlace)
         {
-          (void) CloseBlob(image);
+          if (CloseBlob(image) == MagickFalse)
+            break;
           AppendImageFormat("V",image->filename);
           status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
           if (status == MagickFalse)
             {
+              scanline=(unsigned char *) RelinquishMagickMemory(scanline); 
               image=DestroyImageList(image);
               return((Image *) NULL);
             }
         }
-      for (y=0; y < (long) chroma_image->rows; y++)
+      for (y=0; y < (ssize_t) chroma_image->rows; y++)
       {
-        count=ReadBlob(image,(size_t) chroma_image->columns,scanline);
+        length=quantum*chroma_image->columns;
+        count=ReadBlob(image,length,scanline);
+        if (count != (ssize_t) length)
+          {
+            status=MagickFalse;
+            ThrowFileException(exception,CorruptImageError,
+              "UnexpectedEndOfFile",image->filename);
+            break;
+          }
         p=scanline;
-        q=GetAuthenticPixels(chroma_image,0,y,chroma_image->columns,1,exception);
-        if (q == (PixelPacket *) NULL)
+        q=GetAuthenticPixels(chroma_image,0,y,chroma_image->columns,1,
+          exception);
+        if (q == (Quantum *) NULL)
           break;
-        for (x=0; x < (long) chroma_image->columns; x++)
+        for (x=0; x < (ssize_t) chroma_image->columns; x++)
         {
-          q->blue=ScaleCharToQuantum(*p++);
-          q++;
+          if (quantum == 1)
+            SetPixelBlue(chroma_image,ScaleCharToQuantum(*p++),q);
+          else
+            {
+              SetPixelBlue(chroma_image,ScaleShortToQuantum(((*p) << 8) |
+                *(p+1)),q);
+              p+=(ptrdiff_t) 2;
+            }
+          q+=(ptrdiff_t) GetPixelChannels(chroma_image);
         }
         if (SyncAuthenticPixels(chroma_image,exception) == MagickFalse)
           break;
@@ -354,33 +446,40 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Scale image.
     */
     resize_image=ResizeImage(chroma_image,image->columns,image->rows,
-      TriangleFilter,1.0,exception);
+      TriangleFilter,exception);
     chroma_image=DestroyImage(chroma_image);
     if (resize_image == (Image *) NULL)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-    for (y=0; y < (long) image->rows; y++)
+      {
+        scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+      }
+    for (y=0; y < (ssize_t) image->rows; y++)
     {
+      const Quantum
+        *chroma_pixels;
+
       q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
       chroma_pixels=GetVirtualPixels(resize_image,0,y,resize_image->columns,1,
-        &resize_image->exception);
-      if ((q == (PixelPacket *) NULL) ||
-          (chroma_pixels == (const PixelPacket *) NULL))
+        exception);
+      if ((q == (Quantum *) NULL) ||
+          (chroma_pixels == (const Quantum *) NULL))
         break;
-      for (x=0; x < (long) image->columns; x++)
+      for (x=0; x < (ssize_t) image->columns; x++)
       {
-        q->green=chroma_pixels->green;
-        q->blue=chroma_pixels->blue;
-        chroma_pixels++;
-        q++;
+        SetPixelGreen(image,GetPixelGreen(resize_image,chroma_pixels),q);
+        SetPixelBlue(image,GetPixelBlue(resize_image,chroma_pixels),q);
+        chroma_pixels+=(ptrdiff_t) GetPixelChannels(resize_image);
+        q+=(ptrdiff_t) GetPixelChannels(image);
       }
       if (SyncAuthenticPixels(image,exception) == MagickFalse)
         break;
     }
     resize_image=DestroyImage(resize_image);
-    image->colorspace=YCbCrColorspace;
+    if (SetImageColorspace(image,YCbCrColorspace,exception) == MagickFalse)
+      break;
     if (interlace == PartitionInterlace)
       (void) CopyMagickString(image->filename,image_info->filename,
-        MaxTextExtent);
+        MagickPathExtent);
     if (EOFBlob(image) != MagickFalse)
       {
         ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
@@ -394,19 +493,19 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     if (interlace == NoInterlace)
-      count=ReadBlob(image,(size_t) (2*image->columns),scanline);
+      count=ReadBlob(image,(size_t) (2*quantum*image->columns),scanline);
     else
-      count=ReadBlob(image,(size_t) image->columns,scanline);
+      count=ReadBlob(image,(size_t) quantum*image->columns,scanline);
     if (count != 0)
       {
         /*
           Allocate next image structure.
         */
-        AcquireNextImage(image_info,image);
+        AcquireNextImage(image_info,image,exception);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
-            image=DestroyImageList(image);
-            return((Image *) NULL);
+            status=MagickFalse;
+            break;
           }
         image=SyncNextImageInList(image);
         status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
@@ -416,7 +515,10 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
   } while (count != 0);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
-  (void) CloseBlob(image);
+  if (CloseBlob(image) == MagickFalse)
+    status=MagickFalse;
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   return(GetFirstImageInList(image));
 }
 
@@ -440,21 +542,19 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterYUVImage method is:
 %
-%      unsigned long RegisterYUVImage(void)
+%      size_t RegisterYUVImage(void)
 %
 */
-ModuleExport unsigned long RegisterYUVImage(void)
+ModuleExport size_t RegisterYUVImage(void)
 {
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("YUV");
+  entry=AcquireMagickInfo("YUV","YUV","CCIR 601 4:1:1 or 4:2:2");
   entry->decoder=(DecodeImageHandler *) ReadYUVImage;
   entry->encoder=(EncodeImageHandler *) WriteYUVImage;
-  entry->adjoin=MagickFalse;
-  entry->raw=MagickTrue;
-  entry->description=ConstantString("CCIR 601 4:1:1 or 4:2:2");
-  entry->module=ConstantString("YUV");
+  entry->flags^=CoderAdjoinFlag;
+  entry->flags|=CoderRawSupportFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -500,7 +600,8 @@ ModuleExport void UnregisterYUVImage(void)
 %
 %  The format of the WriteYUVImage method is:
 %
-%      MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteYUVImage(const ImageInfo *image_info,
+%        Image *image,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -508,9 +609,11 @@ ModuleExport void UnregisterYUVImage(void)
 %
 %    o image:  The image.
 %
+%    o exception: return any errors or warnings in this structure.
 %
 */
-static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image,
+  ExceptionInfo *exception)
 {
   Image
     *chroma_image,
@@ -519,34 +622,37 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
   InterlaceType
     interlace;
 
-  long
-    horizontal_factor,
-    vertical_factor,
-    y;
-
   MagickBooleanType
     status;
 
   MagickOffsetType
     scene;
 
-  register const PixelPacket
+  const Quantum
     *p,
     *s;
 
-  register long
+  ssize_t
     x;
 
-  unsigned long
+  size_t
     height,
+    number_scenes,
+    quantum,
     width;
 
+  ssize_t
+    horizontal_factor,
+    vertical_factor,
+    y;
+
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
+  assert(image->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  quantum=(size_t) (image->depth <= 8 ? 1 : 2);
   interlace=image->interlace;
   horizontal_factor=2;
   vertical_factor=2;
@@ -559,12 +665,13 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
         flags;
 
       flags=ParseGeometry(image_info->sampling_factor,&geometry_info);
-      horizontal_factor=(long) geometry_info.rho;
-      vertical_factor=(long) geometry_info.sigma;
-      if ((flags & SigmaValue) == 0)
-        vertical_factor=horizontal_factor;
-      if ((horizontal_factor != 1) && (horizontal_factor != 2) &&
-          (vertical_factor != 1) && (vertical_factor != 2))
+      if ((flags & RhoValue) != 0)
+        horizontal_factor=(ssize_t) geometry_info.rho;
+      vertical_factor=horizontal_factor;
+      if ((flags & SigmaValue) != 0)
+        vertical_factor=(ssize_t) geometry_info.sigma;
+      if (((horizontal_factor != 1) && (horizontal_factor != 2)) ||
+          ((vertical_factor != 1) && (vertical_factor != 2)))
         ThrowWriterException(CorruptImageError,"UnexpectedSamplingFactor");
     }
   if ((interlace == UndefinedInterlace) ||
@@ -579,68 +686,93 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
       /*
         Open output image file.
       */
-      status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+      status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
       if (status == MagickFalse)
         return(status);
     }
   else
     {
       AppendImageFormat("Y",image->filename);
-      status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+      status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
       if (status == MagickFalse)
         return(status);
     }
   scene=0;
+  number_scenes=GetImageListLength(image);
   do
   {
     /*
       Sample image to an even width and height, if necessary.
     */
-    image->depth=8;
-    width=image->columns+(image->columns & (horizontal_factor-1));
-    height=image->rows+(image->rows & (vertical_factor-1));
-    yuv_image=ResizeImage(image,width,height,TriangleFilter,1.0,
-      &image->exception);
+    image->depth=(size_t) (quantum == 1 ? 8 : 16);
+    width=image->columns+(image->columns & (unsigned int) (horizontal_factor
+      -1));
+    height=image->rows+(image->rows & (unsigned int) (vertical_factor-1));
+    yuv_image=ResizeImage(image,width,height,TriangleFilter,exception);
     if (yuv_image == (Image *) NULL)
-      ThrowWriterException(ResourceLimitError,image->exception.reason);
-    (void) SetImageColorspace(yuv_image,YCbCrColorspace);
+      {
+        (void) CloseBlob(image);
+        return(MagickFalse);
+      }
+    (void) TransformImageColorspace(yuv_image,YCbCrColorspace,exception);
     /*
       Downsample image.
     */
-    chroma_image=ResizeImage(image,width/horizontal_factor,
-      height/vertical_factor,TriangleFilter,1.0,&image->exception);
+    chroma_image=ResizeImage(image,(size_t) ((ssize_t) width/horizontal_factor),
+      (size_t) ((ssize_t) height/vertical_factor),TriangleFilter,exception);
     if (chroma_image == (Image *) NULL)
-      ThrowWriterException(ResourceLimitError,image->exception.reason);
-    (void) SetImageColorspace(chroma_image,YCbCrColorspace);
+      {
+        (void) CloseBlob(image);
+        return(MagickFalse);
+      }
+    (void) TransformImageColorspace(chroma_image,YCbCrColorspace,exception);
     if (interlace == NoInterlace)
       {
         /*
           Write noninterlaced YUV.
         */
-        for (y=0; y < (long) yuv_image->rows; y++)
+        for (y=0; y < (ssize_t) yuv_image->rows; y++)
         {
-          p=GetVirtualPixels(yuv_image,0,y,yuv_image->columns,1,
-            &yuv_image->exception);
-          if (p == (const PixelPacket *) NULL)
+          p=GetVirtualPixels(yuv_image,0,y,yuv_image->columns,1,exception);
+          if (p == (const Quantum *) NULL)
             break;
           s=GetVirtualPixels(chroma_image,0,y,chroma_image->columns,1,
-            &chroma_image->exception);
-          if (s == (const PixelPacket *) NULL)
+            exception);
+          if (s == (const Quantum *) NULL)
             break;
-          for (x=0; x < (long) yuv_image->columns; x++)
+          for (x=0; x < (ssize_t) yuv_image->columns; x+=2)
           {
-            (void) WriteBlobByte(image,ScaleQuantumToChar(s->green));
-            (void) WriteBlobByte(image,ScaleQuantumToChar(p->red));
-            p++;
-            (void) WriteBlobByte(image,ScaleQuantumToChar(s->blue));
-            (void) WriteBlobByte(image,ScaleQuantumToChar(p->red));
-            p++;
+            if (quantum == 1)
+              {
+                (void) WriteBlobByte(image,ScaleQuantumToChar(
+                  GetPixelGreen(yuv_image,s)));
+                (void) WriteBlobByte(image,ScaleQuantumToChar(
+                  GetPixelRed(yuv_image,p)));
+                p+=(ptrdiff_t) GetPixelChannels(yuv_image);
+                (void) WriteBlobByte(image,ScaleQuantumToChar(
+                  GetPixelBlue(yuv_image,s)));
+                (void) WriteBlobByte(image,ScaleQuantumToChar(
+                  GetPixelRed(yuv_image,p)));
+              }
+            else
+              {
+                (void) WriteBlobByte(image,ScaleQuantumToChar(
+                  GetPixelGreen(yuv_image,s)));
+                (void) WriteBlobShort(image,ScaleQuantumToShort(
+                  GetPixelRed(yuv_image,p)));
+                p+=(ptrdiff_t) GetPixelChannels(yuv_image);
+                (void) WriteBlobByte(image,ScaleQuantumToChar(
+                  GetPixelBlue(yuv_image,s)));
+                (void) WriteBlobShort(image,ScaleQuantumToShort(
+                  GetPixelRed(yuv_image,p)));
+              }
+            p+=(ptrdiff_t) GetPixelChannels(yuv_image);
             s++;
-            x++;
           }
           if (image->previous == (Image *) NULL)
             {
-              status=SetImageProgress(image,SaveImageTag,y,image->rows);
+              status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+                image->rows);
               if (status == MagickFalse)
                 break;
             }
@@ -652,20 +784,25 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
         /*
           Initialize Y channel.
         */
-        for (y=0; y < (long) yuv_image->rows; y++)
+        for (y=0; y < (ssize_t) yuv_image->rows; y++)
         {
-          p=GetVirtualPixels(yuv_image,0,y,yuv_image->columns,1,
-            &yuv_image->exception);
-          if (p == (const PixelPacket *) NULL)
+          p=GetVirtualPixels(yuv_image,0,y,yuv_image->columns,1,exception);
+          if (p == (const Quantum *) NULL)
             break;
-          for (x=0; x < (long) yuv_image->columns; x++)
+          for (x=0; x < (ssize_t) yuv_image->columns; x++)
           {
-            (void) WriteBlobByte(image,ScaleQuantumToChar(p->red));
-            p++;
+            if (quantum == 1)
+              (void) WriteBlobByte(image,ScaleQuantumToChar(
+                GetPixelRed(yuv_image,p)));
+            else
+              (void) WriteBlobShort(image,ScaleQuantumToShort(
+                GetPixelRed(yuv_image,p)));
+            p+=(ptrdiff_t) GetPixelChannels(yuv_image);
           }
           if (image->previous == (Image *) NULL)
             {
-              status=SetImageProgress(image,SaveImageTag,y,image->rows);
+              status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+                image->rows);
               if (status == MagickFalse)
                 break;
             }
@@ -682,23 +819,28 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
         */
         if (interlace == PartitionInterlace)
           {
-            (void) CloseBlob(image);
+            if (CloseBlob(image) == MagickFalse)
+              break;
             AppendImageFormat("U",image->filename);
-            status=OpenBlob(image_info,image,WriteBinaryBlobMode,
-              &image->exception);
+            status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
             if (status == MagickFalse)
               return(status);
           }
-        for (y=0; y < (long) chroma_image->rows; y++)
+        for (y=0; y < (ssize_t) chroma_image->rows; y++)
         {
           p=GetVirtualPixels(chroma_image,0,y,chroma_image->columns,1,
-            &chroma_image->exception);
-          if (p == (const PixelPacket *) NULL)
+            exception);
+          if (p == (const Quantum *) NULL)
             break;
-          for (x=0; x < (long) chroma_image->columns; x++)
+          for (x=0; x < (ssize_t) chroma_image->columns; x++)
           {
-            (void) WriteBlobByte(image,ScaleQuantumToChar(p->green));
-            p++;
+            if (quantum == 1)
+              (void) WriteBlobByte(image,ScaleQuantumToChar(
+                GetPixelGreen(chroma_image,p)));
+            else
+              (void) WriteBlobShort(image,ScaleQuantumToShort(
+                GetPixelGreen(chroma_image,p)));
+            p+=(ptrdiff_t) GetPixelChannels(chroma_image);
           }
         }
         if (image->previous == (Image *) NULL)
@@ -712,23 +854,28 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
         */
         if (interlace == PartitionInterlace)
           {
-            (void) CloseBlob(image);
+            if (CloseBlob(image) == MagickFalse)
+              break;
             AppendImageFormat("V",image->filename);
-            status=OpenBlob(image_info,image,WriteBinaryBlobMode,
-              &image->exception);
+            status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
             if (status == MagickFalse)
               return(status);
           }
-        for (y=0; y < (long) chroma_image->rows; y++)
+        for (y=0; y < (ssize_t) chroma_image->rows; y++)
         {
           p=GetVirtualPixels(chroma_image,0,y,chroma_image->columns,1,
-            &chroma_image->exception);
-          if (p == (const PixelPacket *) NULL)
+            exception);
+          if (p == (const Quantum *) NULL)
             break;
-          for (x=0; x < (long) chroma_image->columns; x++)
+          for (x=0; x < (ssize_t) chroma_image->columns; x++)
           {
-            (void) WriteBlobByte(image,ScaleQuantumToChar(p->blue));
-            p++;
+            if (quantum == 1)
+              (void) WriteBlobByte(image,ScaleQuantumToChar(
+                GetPixelBlue(chroma_image,p)));
+            else
+              (void) WriteBlobShort(image,ScaleQuantumToShort(
+                GetPixelBlue(chroma_image,p)));
+            p+=(ptrdiff_t) GetPixelChannels(chroma_image);
           }
         }
         if (image->previous == (Image *) NULL)
@@ -741,15 +888,15 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
     chroma_image=DestroyImage(chroma_image);
     if (interlace == PartitionInterlace)
       (void) CopyMagickString(image->filename,image_info->filename,
-        MaxTextExtent);
+        MagickPathExtent);
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=SetImageProgress(image,SaveImagesTag,scene++,
-      GetImageListLength(image));
+    status=SetImageProgress(image,SaveImagesTag,scene++,number_scenes);
     if (status == MagickFalse)
       break;
   } while (image_info->adjoin != MagickFalse);
-  (void) CloseBlob(image);
-  return(MagickTrue);
+  if (CloseBlob(image) == MagickFalse)
+    status=MagickFalse;
+  return(status);
 }
